@@ -5,7 +5,7 @@
 /*  The code for generating a calendar.                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -15,6 +15,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <ctype.h>
 
 #include <stdlib.h>
@@ -26,6 +27,11 @@
 #include <wchar.h>
 #endif
 
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+
+#include "lang.h"
 #include "types.h"
 #include "protos.h"
 #include "expr.h"
@@ -75,6 +81,8 @@ static struct line_drawing VT100Drawing = {
   "\x6c", "\x74", "\x6a", "\x71"
 };
 
+static int encoding_is_utf8 = 0;
+
 static struct line_drawing UTF8Drawing = {
     "", "",
     "\xe2\x94\xb4",
@@ -248,6 +256,17 @@ static void WriteBottomCalLine (void);
 static void WriteIntermediateCalLine (void);
 static void WriteCalDays (void);
 
+static void
+send_lrm(void)
+{
+    /* Send a lrm control sequence if UseUTF8Chars is enabled
+       or char encoding is UTF-8
+    */
+    if (UseUTF8Chars || encoding_is_utf8) {
+        printf("\xE2\x80\x8E");
+    }
+}
+
 static char const *
 despace(char const *s)
 {
@@ -587,6 +606,14 @@ void ProduceCalendar(void)
 {
     int y, m, d;
 
+    /* Check if current locale is UTF-8, if we have langinfo.h */
+#ifdef HAVE_LANGINFO_H
+    char const *encoding = nl_langinfo(CODESET);
+    if (!strcasecmp(encoding, "utf-8")) {
+        encoding_is_utf8 = 1;
+    }
+
+#endif
     if (UseUTF8Chars) {
 	linestruct = &UTF8Drawing;
     } else if (UseVTChars) {
@@ -975,6 +1002,10 @@ static void PrintLeft(char const *s, int width, char pad)
     while (*ws && wcwidth(*ws) == 0) {
         PutWideChar(*ws++);
     }
+
+    /* Possibly send lrm control sequence */
+    send_lrm();
+
     for (i=display_len; i<width; i++) fputc(pad, stdout);
     if (buf != static_buf) free(buf);
 #endif
@@ -1055,6 +1086,9 @@ static void PrintCentered(char const *s, int width, char *pad)
     while (*ws && wcwidth(*ws) == 0) {
         PutWideChar(*ws++);
     }
+    /* Possibly send lrm control sequence */
+    send_lrm();
+
     for (i=d+display_len; i<width; i++) fputs(pad, stdout);
     if (buf != static_buf) free(buf);
 #endif
@@ -1089,7 +1123,6 @@ static int WriteOneCalLine(void)
     return done;
 }
 
-
 /***************************************************************/
 /*                                                             */
 /*  WriteOneColLine                                            */
@@ -1140,9 +1173,14 @@ static int WriteOneColLine(int col)
             }
 	    if (iswspace(*ws)) {
                 wspace = ws;
-            }
-            if (wcwidth(*ws)) {
                 width++;
+            } else {
+                if (wcwidth(*ws)) {
+                    width += wcwidth(*ws);
+                    if (width > ColSpaces) {
+                        break;
+                    }
+                }
             }
 	    ws++;
 	}
@@ -1154,13 +1192,16 @@ static int WriteOneColLine(int col)
 
 	/* If we couldn't find a space char, print what we have. */
 	if (!wspace) {
-	    for (ws = e->wc_pos; ws - e->wc_pos < ColSpaces; ws++) {
+	    for (ws = e->wc_pos; numwritten < ColSpaces; ws++) {
 		if (!*ws) break;
                 if (iswspace(*ws)) {
                     putchar(' ');
                     numwritten++;
                 } else {
                     if (wcwidth(*ws) > 0) {
+                        if (numwritten + wcwidth(*ws) > ColSpaces) {
+                            break;
+                        }
                         numwritten += wcwidth(*ws);
                     }
                     PutWideChar(*ws);
@@ -1188,6 +1229,9 @@ static int WriteOneColLine(int col)
 	    printf("%s", Decolorize(e->r, e->g, e->b));
 	}
 
+        /* Possibly send lrm control sequence */
+        send_lrm();
+
 	/* Flesh out the rest of the column */
 	while(numwritten++ < ColSpaces) putchar(' ');
 
@@ -1824,6 +1868,13 @@ static int DoCalRem(ParsePtr p, int col)
 	e->next = CurCol;
 	CalColumn[col] = e;
 	SortCol(&CalColumn[col]);
+    } else {
+        /* Parse the rest of the line to catch expression-pasting errors */
+        while (ParseChar(p, &r, 0)) {
+            if (r != 0) {
+                return r;
+            }
+        }
     }
     return OK;
 }
@@ -2195,14 +2246,14 @@ CalendarTime(int tim, int duration)
     }
 
     if (h >= 12) {
-	ampm1 = L_PM;
+	ampm1 = DynamicPm;
     } else {
-	ampm1 = L_AM;
+	ampm1 = DynamicAm;
     }
     if (h2 >= 12) {
-	ampm2 = L_PM;
+	ampm2 = DynamicPm;
     } else {
-	ampm2 = L_AM;
+	ampm2 = DynamicAm;
     }
     if (!days) {
 	if (!strcmp(ampm1, ampm2)) {
@@ -2249,7 +2300,7 @@ char const *SimpleTime(int tim)
 	    if (h == 0) hh=12;
 	    else if (h > 12) hh=h-12;
 	    else hh=h;
-	    sprintf(buf, "%d%c%02d%s ", hh, TimeSep, min, (h>=12) ? L_PM : L_AM);
+	    sprintf(buf, "%d%c%02d%s ", hh, TimeSep, min, (h>=12) ? DynamicPm : DynamicAm);
 	}
 	break;
 
