@@ -7,7 +7,7 @@
 /*  commands.                                                  */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -28,7 +28,7 @@ static int ParseTimeTrig (ParsePtr s, TimeTrig *tim, int save_in_globals);
 static int ParseLocalOmit (ParsePtr s, Trigger *t);
 static int ParseScanFrom (ParsePtr s, Trigger *t, int type);
 static int ParsePriority (ParsePtr s, Trigger *t);
-static int ParseUntil (ParsePtr s, Trigger *t);
+static int ParseUntil (ParsePtr s, Trigger *t, int type);
 static int ShouldTriggerBasedOnWarn (Trigger *t, int jul, int *err);
 static int ComputeTrigDuration(TimeTrig *t);
 
@@ -168,27 +168,37 @@ int DoRem(ParsePtr p)
 	FreeTrig(&trig);
 	return OK;
     }
-/* Queue the reminder, if necessary */
+
+    /* Queue the reminder, if necessary */
     if (jul == JulianToday &&
 	!(!IgnoreOnce &&
 	  trig.once != NO_ONCE &&
 	  FileAccessDate == JulianToday))
 	QueueReminder(p, &trig, &tim, trig.sched);
-/* If we're in daemon mode, do nothing over here */
+    /* If we're in daemon mode, do nothing over here */
     if (Daemon) {
 	FreeTrig(&trig);
 	return OK;
     }
 
+    r = OK;
     if (ShouldTriggerReminder(&trig, &tim, jul, &err)) {
 	if ( (r=TriggerReminder(p, &trig, &tim, jul)) ) {
 	    FreeTrig(&trig);
 	    return r;
 	}
+    } else {
+        /* Parse the rest of the line to catch any potential
+           expression-pasting errors */
+        while (ParseChar(p, &r, 0)) {
+            if (r != 0) {
+                break;
+            }
+        }
     }
 
     FreeTrig(&trig);
-    return OK;
+    return r;
 }
 
 /***************************************************************/
@@ -237,11 +247,15 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
     tim->delta = DefaultTDelta;
     tim->rep   = NO_REP;
     tim->duration = NO_TIME;
+    trig->need_wkday = 0;
+    trig->adj_for_last = 0;
+
     if (save_in_globals) {
 	LastTriggerTime = NO_TIME;
     }
 
-    while(1) {
+    int parsing = 1;
+    while(parsing) {
 	/* Read space-delimited string */
 	r = ParseToken(s, &buf);
 	if (r) return r;
@@ -249,6 +263,25 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	/* Figure out what we've got */
 	FindToken(DBufValue(&buf), &tok);
 	switch(tok.type) {
+        case T_In:
+            /* Completely ignored */
+	    DBufFree(&buf);
+            break;
+
+        case T_Ordinal:
+            DBufFree(&buf);
+            if (trig->d != NO_DAY)     return E_DAY_TWICE;
+            if (tok.val < 0) {
+                if (trig->back != NO_BACK) return E_BACK_TWICE;
+                trig->back = -7;
+                trig->d = 1;
+                trig->adj_for_last = 1;
+            } else {
+                trig->d = 1 + 7 * tok.val;
+            }
+            trig->need_wkday = 1;
+            break;
+
 	case T_Date:
 	    DBufFree(&buf);
 	    if (trig->d != NO_DAY) return E_DAY_TWICE;
@@ -322,7 +355,6 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    DBufFree(&buf);
 	    trig->typ = tok.val;
 	    if (s->isnested) return E_CANT_NEST_RTYPE;
-	    if (trig->scanfrom == NO_DATE) trig->scanfrom = JulianToday;
 	    if (trig->typ == PASSTHRU_TYPE) {
 		r = ParseToken(s, &buf);
 		if (r) return r;
@@ -332,19 +364,20 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 		}
 		StrnCpy(trig->passthru, DBufValue(&buf), PASSTHRU_LEN);
 	    }
-	    return OK;
+            parsing = 0;
+            break;
 
 	case T_Through:
 	    DBufFree(&buf);
 	    if (trig->rep != NO_REP) return E_REP_TWICE;
 	    trig->rep = 1;
-	    r = ParseUntil(s, trig);
+	    r = ParseUntil(s, trig, tok.type);
 	    if (r) return r;
 	    break;
 
 	case T_Until:
 	    DBufFree(&buf);
-	    r=ParseUntil(s, trig);
+	    r=ParseUntil(s, trig, tok.type);
 	    if (r) return r;
 	    break;
 
@@ -378,6 +411,15 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    trig->back = tok.val;
 	    break;
 
+	case T_BackAdj:
+	    DBufFree(&buf);
+	    if (trig->back != NO_BACK) return E_BACK_TWICE;
+	    if (trig->d != NO_DAY) return E_DAY_TWICE;
+	    trig->back = tok.val;
+            trig->d = 1;
+            trig->adj_for_last = 1;
+	    break;
+
 	case T_Once:
 	    DBufFree(&buf);
 	    if (trig->once != NO_ONCE) return E_ONCE_TWICE;
@@ -392,7 +434,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	case T_Omit:
 	    DBufFree(&buf);
 	    if (trig->omitfunc[0]) {
-		Eprint("Warning: OMIT is ignored if you use OMITFUNC");
+		Wprint("Warning: OMIT is ignored if you use OMITFUNC");
 	    }
 
 	    r = ParseLocalOmit(s, trig);
@@ -401,12 +443,12 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 
 	case T_Empty:
 	    DBufFree(&buf);
-	    if (trig->scanfrom == NO_DATE) trig->scanfrom = JulianToday;
-	    return OK;
+            parsing = 0;
+            break;
 
 	case T_OmitFunc:
 	    if (trig->localomit) {
-		Eprint("Warning: OMIT is ignored if you use OMITFUNC");
+		Wprint("Warning: OMIT is ignored if you use OMITFUNC");
 	    }
 	    r=ParseToken(s, &buf);
 	    if (r) return r;
@@ -474,10 +516,58 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim, int save_in_globals)
 	    DBufFree(&buf);
 	    trig->typ = MSG_TYPE;
 	    if (s->isnested) return E_CANT_NEST_RTYPE;
-	    if (trig->scanfrom == NO_DATE) trig->scanfrom = JulianToday;
-	    return OK;
+            parsing = 0;
+            break;
 	}
     }
+
+    if (trig->need_wkday && trig->wd == NO_WD) {
+        Eprint("Weekday name(s) required");
+        return E_PARSE_ERR;
+    }
+
+    /* Adjust month and possibly year */
+    if (trig->adj_for_last) {
+        if (trig->m != NO_MON) {
+            trig->m++;
+            if (trig->m >= 12) {
+                trig->m = 0;
+                if (trig->y != NO_YR) {
+                    trig->y++;
+                }
+            }
+        }
+        trig->adj_for_last = 0;
+    }
+
+    /* Check for some warning conditions */
+    if (!s->nonconst_expr) {
+        if (trig->y != NO_YR && trig->m != NO_MON && trig->d != NO_DAY && trig->until != NO_UNTIL) {
+            if (Julian(trig->y, trig->m, trig->d) > trig->until) {
+                Wprint("Warning: UNTIL/THROUGH date earlier than start date");
+            }
+        }
+        if (trig->from != NO_DATE) {
+            if (trig->until != NO_UNTIL && trig->until < trig->from) {
+                Wprint("Warning: UNTIL/THROUGH date earlier than FROM date");
+            }
+        } else if (trig->scanfrom != NO_DATE) {
+            if (trig->until != NO_UNTIL && trig->until < trig->scanfrom) {
+                Wprint("Warning: UNTIL/THROUGH date earlier than SCANFROM date");
+            }
+        }
+    }
+
+    if (trig->y != NO_YR && trig->m != NO_MON && trig->d != NO_DAY && trig->until != NO_UNTIL && trig->rep == NO_REP) {
+        Wprint("Warning: Useless use of UNTIL with fully-specified date and no *rep");
+    }
+
+    /* Set scanfrom to default if not set explicitly */
+    if (trig->scanfrom == NO_DATE) {
+        trig->scanfrom = JulianToday;
+    }
+
+    return OK;
 }
 
 /***************************************************************/
@@ -568,12 +658,18 @@ static int ParseLocalOmit(ParsePtr s, Trigger *t)
 /*  ParseUntil - parse the UNTIL portion of a reminder         */
 /*                                                             */
 /***************************************************************/
-static int ParseUntil(ParsePtr s, Trigger *t)
+static int ParseUntil(ParsePtr s, Trigger *t, int type)
 {
     int y = NO_YR,
 	m = NO_MON,
 	d = NO_DAY;
 
+    char const *which;
+    if (type == T_Until) {
+        which = "UNTIL";
+    } else {
+        which = "THROUGH";
+    }
     Token tok;
     int r;
     DynamicBuffer buf;
@@ -589,7 +685,7 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 	case T_Year:
 	    DBufFree(&buf);
 	    if (y != NO_YR) {
-		Eprint("UNTIL: %s", ErrMsg[E_YR_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_YR_TWICE]);
 		return E_YR_TWICE;
 	    }
 	    y = tok.val;
@@ -598,7 +694,7 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 	case T_Month:
 	    DBufFree(&buf);
 	    if (m != NO_MON) {
-		Eprint("UNTIL: %s", ErrMsg[E_MON_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_MON_TWICE]);
 		return E_MON_TWICE;
 	    }
 	    m = tok.val;
@@ -607,7 +703,7 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 	case T_Day:
 	    DBufFree(&buf);
 	    if (d != NO_DAY) {
-		Eprint("UNTIL: %s", ErrMsg[E_DAY_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_DAY_TWICE]);
 		return E_DAY_TWICE;
 	    }
 	    d = tok.val;
@@ -616,15 +712,15 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 	case T_Date:
 	    DBufFree(&buf);
 	    if (y != NO_YR) {
-		Eprint("UNTIL: %s", ErrMsg[E_YR_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_YR_TWICE]);
 		return E_YR_TWICE;
 	    }
 	    if (m != NO_MON) {
-		Eprint("UNTIL: %s", ErrMsg[E_MON_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_MON_TWICE]);
 		return E_MON_TWICE;
 	    }
 	    if (d != NO_DAY) {
-		Eprint("UNTIL: %s", ErrMsg[E_DAY_TWICE]);
+		Eprint("%s: %s", which, ErrMsg[E_DAY_TWICE]);
 		return E_DAY_TWICE;
 	    }
 	    FromJulian(tok.val, &y, &m, &d);
@@ -632,7 +728,7 @@ static int ParseUntil(ParsePtr s, Trigger *t)
 
 	default:
 	    if (y == NO_YR || m == NO_MON || d == NO_DAY) {
-		Eprint("UNTIL: %s", ErrMsg[E_INCOMPLETE]);
+		Eprint("%s: %s", which, ErrMsg[E_INCOMPLETE]);
 		DBufFree(&buf);
 		return E_INCOMPLETE;
 	    }
