@@ -6,11 +6,12 @@
 /*  reminders are triggered.                                   */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
 #include "config.h"
+#include "expr.h"
 #define L_IN_DOSUBST
 #include <stdio.h>
 #include <string.h>
@@ -29,9 +30,6 @@
 #define NL "\n"
 #endif
 
-static char TODAY[] = L_TODAY;
-static char TOMORROW[] = L_TOMORROW;
-
 #define SHIP_OUT(s) if(DBufPuts(dbuf, s) != OK) return E_NO_MEM
 
 /***************************************************************/
@@ -58,10 +56,17 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
     char const *mplu, *hplu, *when, *plu;
     int has_quote = 0;
     char *ss;
+    char const *expr;
     char *os;
     char s[256];
+    char uf[32];
+    char mypm[64];
+    char mycpm[64];
+    char myplu[64];
     int origLen = DBufLen(dbuf);
     int altmode;
+    int r;
+    Value v;
 
     FromJulian(jul, &y, &m, &d);
 
@@ -74,16 +79,16 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef	L_MPLU_OVER
     L_MPLU_OVER
 #else /* L_MPLU_OVER */
-    mplu = (mdiff == 1 ? "" : L_MPLU);
+    mplu = (mdiff == 1 ? "" : DynamicMplu);
 #endif /* L_MPLU_OVER */
 
 #ifdef L_HPLU_OVER
     L_HPLU_OVER
 #else /* L_HPLU_OVER */
-    hplu = (hdiff == 1 ? "" : L_HPLU);
+    hplu = (hdiff == 1 ? "" : DynamicHplu);
 #endif /* L_HPLU_OVER */
 
-    when = (tdiff < 0 ? L_AGO : L_FROMNOW);
+    when = (tdiff < 0) ? DynamicAgo : DynamicFromnow;
 
     h = tim / 60;
     min = tim % 60;
@@ -91,7 +96,26 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_AMPM_OVERRIDE
     L_AMPM_OVERRIDE (pm, h)
 #else
-    pm = (h < 12) ? L_AM : L_PM;
+    r = -1;
+    if (UserFuncExists("subst_ampm") == 1) {
+        snprintf(s, sizeof(s), "subst_ampm(%d)", h);
+        expr = (char const *) s;
+        r = EvalExpr(&expr, &v, NULL);
+        if (r == OK) {
+            if (!DoCoerce(STR_TYPE, &v)) {
+                snprintf(mypm, sizeof(mypm), "%s", v.v.str);
+                pm = mypm;
+            } else {
+                r = -1;
+            }
+            DestroyValue(v);
+        } else {
+            Eprint("%s", ErrMsg[r]);
+        }
+    }
+    if (r != OK) {
+        pm = (h < 12) ? DynamicAm : DynamicPm;
+    }
 #endif
     hh = (h == 12) ? 12 : h % 12;
 
@@ -101,25 +125,60 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_AMPM_OVERRIDE
     L_AMPM_OVERRIDE (cpm, ch)
 #else
-    cpm = (ch < 12) ? L_AM : L_PM;
+    r = -1;
+    if (UserFuncExists("subst_ampm") == 1) {
+        snprintf(s, sizeof(s), "subst_ampm(%d)", ch);
+        expr = (char const *) s;
+        r = EvalExpr(&expr, &v, NULL);
+        if (r == OK) {
+            if (!DoCoerce(STR_TYPE, &v)) {
+                snprintf(mycpm, sizeof(mycpm), "%s", v.v.str);
+                cpm = mycpm;
+            } else {
+                r = -1;
+            }
+        } else {
+            Eprint("%s", ErrMsg[r]);
+        }
+    }
+    if (r != OK) {
+        cpm = (h < 12) ? DynamicAm : DynamicPm;
+    }
 #endif
     chh = (ch == 12) ? 12 : ch % 12;
 
 #ifdef L_ORDINAL_OVERRIDE
-    L_ORDINAL_OVERRIDE
+    L_ORDINAL_OVERRIDE;
 #else
-    switch(d) {
-    case 1:
-    case 21:
-    case 31: plu = "st"; break;
+    if (UserFuncExists("subst_ordinal") == 1) {
+        snprintf(s, sizeof(s), "subst_ordinal(%d)", d);
+        expr = (char const *) s;
+        r = EvalExpr(&expr, &v, NULL);
+        if (r == OK) {
+            if (!DoCoerce(STR_TYPE, &v)) {
+                snprintf(myplu, sizeof(myplu), "%s", v.v.str);
+                plu = myplu;
+            } else {
+                r = -1;
+            }
+        } else {
+            Eprint("%s", ErrMsg[r]);
+        }
+    }
+    if (r != OK) {
+        switch(d) {
+        case 1:
+        case 21:
+        case 31: plu = "st"; break;
 
-    case 2:
-    case 22: plu = "nd"; break;
+        case 2:
+        case 22: plu = "nd"; break;
 
-    case 3:
-    case 23: plu = "rd"; break;
+        case 3:
+        case 23: plu = "rd"; break;
 
-    default: plu = "th"; break;
+        default: plu = "th"; break;
+        }
     }
 #endif
 
@@ -163,6 +222,29 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    }
 	}
 	done = 0;
+        snprintf(uf, sizeof(uf), "subst_%c", c);
+        if (UserFuncExists(uf) == 3) {
+            snprintf(s, sizeof(s), "subst_%c(%d,'%04d-%02d-%02d',%02d:%02d)",
+                     c, altmode ? 1 : 0, y, m+1, d, h, min);
+            expr = (char const *) s;
+            r = EvalExpr(&expr, &v, NULL);
+            if (r == OK) {
+                if (v.type != INT_TYPE || v.v.val != 0) {
+                    if (!DoCoerce(STR_TYPE, &v)) {
+                        if (DBufPuts(dbuf, v.v.str) != OK) {
+                            DestroyValue(v);
+                            return E_NO_MEM;
+                        }
+                    }
+                    DestroyValue(v);
+                    continue;
+                }
+                DestroyValue(v);
+            } else {
+                Eprint("%s", ErrMsg[r]);
+            }
+        }
+
 	if (diff <= 1) {
 	    switch(UPPER(c)) {
 #ifndef L_NOTOMORROW_A
@@ -204,7 +286,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifndef L_NOTOMORROW_V
 	    case 'V':
 #endif
-		sprintf(s, "%s", (diff ? TOMORROW : TODAY));
+		snprintf(s, sizeof(s), "%s", (diff ? DynamicTomorrow: DynamicToday));
 		SHIP_OUT(s);
 		done = 1;
 		break;
@@ -213,16 +295,42 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    }
 	}
 
+
+        if (!done) {
+            snprintf(uf, sizeof(uf), "subst_%cx", c);
+            if (UserFuncExists(uf) == 3) {
+                snprintf(s, sizeof(s), "subst_%cx(%d,'%04d-%02d-%02d',%02d:%02d)",
+                         c, altmode ? 1 : 0, y, m+1, d, h, min);
+                expr = (char const *) s;
+                r = EvalExpr(&expr, &v, NULL);
+                if (r == OK) {
+                    if (v.type != INT_TYPE || v.v.val != 0) {
+                        if (!DoCoerce(STR_TYPE, &v)) {
+                            if (DBufPuts(dbuf, v.v.str) != OK) {
+                                DestroyValue(v);
+                                return E_NO_MEM;
+                            }
+                        }
+                        DestroyValue(v);
+                        continue;
+                    }
+                    DestroyValue(v);
+                } else {
+                    Eprint("%s", ErrMsg[r]);
+                }
+            }
+        }
+
 	if (!done) switch(UPPER(c)) {
 	case 'A':
 #ifdef L_A_OVER
 	    L_A_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %d %s, %d", get_day_name(jul%7), d,
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %d %s, %d", get_day_name(jul%7), d,
 			get_month_name(m), y);
 	    } else {
-		sprintf(s, "%s %s, %d %s, %d", L_ON, get_day_name(jul%7), d,
+		snprintf(s, sizeof(s), "%s %s, %d %s, %d", DynamicOn, get_day_name(jul%7), d,
 			get_month_name(m), y);
 	    }
 #endif
@@ -233,7 +341,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_B_OVER
 	    L_B_OVER
 #else
-	    sprintf(s, L_INXDAYS, diff);
+	    snprintf(s, sizeof(s), L_INXDAYS, diff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -242,10 +350,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_C_OVER
 	    L_C_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s", get_day_name(jul%7));
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s", get_day_name(jul%7));
 	    } else {
-		sprintf(s, "%s %s", L_ON, get_day_name(jul%7));
+		snprintf(s, sizeof(s), "%s %s", DynamicOn, get_day_name(jul%7));
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -255,7 +363,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_D_OVER
 	    L_D_OVER
 #else
-	    sprintf(s, "%d", d);
+	    snprintf(s, sizeof(s), "%d", d);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -264,11 +372,11 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_E_OVER
 	    L_E_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%02d%c%02d%c%04d", d, DateSep,
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%02d%c%02d%c%04d", d, DateSep,
 			m+1, DateSep, y);
 	    } else {
-		sprintf(s, "%s %02d%c%02d%c%04d", L_ON, d, DateSep,
+		snprintf(s, sizeof(s), "%s %02d%c%02d%c%04d", DynamicOn, d, DateSep,
 			m+1, DateSep, y);
 	    }
 #endif
@@ -279,10 +387,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_F_OVER
 	    L_F_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%02d%c%02d%c%04d", m+1, DateSep, d, DateSep, y);
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%02d%c%02d%c%04d", m+1, DateSep, d, DateSep, y);
 	    } else {
-		sprintf(s, "%s %02d%c%02d%c%04d", L_ON, m+1, DateSep, d, DateSep, y);
+		snprintf(s, sizeof(s), "%s %02d%c%02d%c%04d", DynamicOn, m+1, DateSep, d, DateSep, y);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -292,10 +400,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_G_OVER
 	    L_G_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %d %s", get_day_name(jul%7), d, get_month_name(m));
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %d %s", get_day_name(jul%7), d, get_month_name(m));
 	    } else {
-		sprintf(s, "%s %s, %d %s", L_ON, get_day_name(jul%7), d, get_month_name(m));
+		snprintf(s, sizeof(s), "%s %s, %d %s", DynamicOn, get_day_name(jul%7), d, get_month_name(m));
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -305,10 +413,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_H_OVER
 	    L_H_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%02d%c%02d", d, DateSep, m+1);
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%02d%c%02d", d, DateSep, m+1);
 	    } else {
-		sprintf(s, "%s %02d%c%02d", L_ON, d, DateSep, m+1);
+		snprintf(s, sizeof(s), "%s %02d%c%02d", DynamicOn, d, DateSep, m+1);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -318,10 +426,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_I_OVER
 	    L_I_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%02d%c%02d", m+1, DateSep, d);
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%02d%c%02d", m+1, DateSep, d);
 	    } else {
-		sprintf(s, "%s %02d%c%02d", L_ON, m+1, DateSep, d);
+		snprintf(s, sizeof(s), "%s %02d%c%02d", DynamicOn, m+1, DateSep, d);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -331,11 +439,11 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_J_OVER
 	    L_J_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %s %d%s, %d", get_day_name(jul%7),
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %s %d%s, %d", get_day_name(jul%7),
 			get_month_name(m), d, plu, y);
 	    } else {
-		sprintf(s, "%s %s, %s %d%s, %d", L_ON, get_day_name(jul%7),
+		snprintf(s, sizeof(s), "%s %s, %s %d%s, %d", DynamicOn, get_day_name(jul%7),
 			get_month_name(m), d, plu, y);
 	    }
 #endif
@@ -346,11 +454,11 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_K_OVER
 	    L_K_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %s %d%s", get_day_name(jul%7),
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %s %d%s", get_day_name(jul%7),
 			get_month_name(m), d, plu);
 	    } else {
-		sprintf(s, "%s %s, %s %d%s", L_ON, get_day_name(jul%7),
+		snprintf(s, sizeof(s), "%s %s, %s %d%s", DynamicOn, get_day_name(jul%7),
 			get_month_name(m), d, plu);
 	    }
 #endif
@@ -361,10 +469,10 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_L_OVER
 	    L_L_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%04d%c%02d%c%02d", y, DateSep, m+1, DateSep, d);
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%04d%c%02d%c%02d", y, DateSep, m+1, DateSep, d);
 	    } else {
-		sprintf(s, "%s %04d%c%02d%c%02d", L_ON, y, DateSep, m+1, DateSep, d);
+		snprintf(s, sizeof(s), "%s %04d%c%02d%c%02d", DynamicOn, y, DateSep, m+1, DateSep, d);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -374,7 +482,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_M_OVER
 	    L_M_OVER
 #else
-	    sprintf(s, "%s", get_month_name(m));
+	    snprintf(s, sizeof(s), "%s", get_month_name(m));
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -383,7 +491,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_N_OVER
 	    L_N_OVER
 #else
-	    sprintf(s, "%d", m+1);
+	    snprintf(s, sizeof(s), "%d", m+1);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -392,7 +500,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_O_OVER
 	    L_O_OVER
 #else
-	    if (RealToday == JulianToday) sprintf(s, " (%s)", L_TODAY);
+            if (RealToday == JulianToday) snprintf(s, sizeof(s), " (%s)", DynamicToday);
 	    else *s = 0;
 #endif
 	    SHIP_OUT(s);
@@ -402,7 +510,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_P_OVER
 	    L_P_OVER
 #else
-	    sprintf(s, "%s", (diff == 1 ? "" : L_PLURAL));
+	    snprintf(s, sizeof(s), "%s", (diff == 1 ? "" : L_PLURAL));
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -411,7 +519,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_Q_OVER
 	    L_Q_OVER
 #else
-	    sprintf(s, "%s", (diff == 1 ? "'s" : "s'"));
+	    snprintf(s, sizeof(s), "%s", (diff == 1 ? "'s" : "s'"));
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -420,7 +528,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_R_OVER
 	    L_R_OVER
 #else
-	    sprintf(s, "%02d", d);
+	    snprintf(s, sizeof(s), "%02d", d);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -429,7 +537,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_S_OVER
 	    L_S_OVER
 #else
-	    sprintf(s, "%s", plu);
+	    snprintf(s, sizeof(s), "%s", plu);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -438,7 +546,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_T_OVER
 	    L_T_OVER
 #else
-	    sprintf(s, "%02d", m+1);
+	    snprintf(s, sizeof(s), "%02d", m+1);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -447,11 +555,11 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_U_OVER
 	    L_U_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %d%s %s, %d", get_day_name(jul%7), d,
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %d%s %s, %d", get_day_name(jul%7), d,
 			plu, get_month_name(m), y);
 	    } else {
-		sprintf(s, "%s %s, %d%s %s, %d", L_ON, get_day_name(jul%7), d,
+		snprintf(s, sizeof(s), "%s %s, %d%s %s, %d", DynamicOn, get_day_name(jul%7), d,
 			plu, get_month_name(m), y);
 	    }
 #endif
@@ -462,11 +570,11 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_V_OVER
 	    L_V_OVER
 #else
-	    if (altmode == '*') {
-		sprintf(s, "%s, %d%s %s", get_day_name(jul%7), d, plu,
+            if (altmode == '*' || !strcmp(DynamicOn, "")) {
+		snprintf(s, sizeof(s), "%s, %d%s %s", get_day_name(jul%7), d, plu,
 			get_month_name(m));
 	    } else {
-		sprintf(s, "%s %s, %d%s %s", L_ON, get_day_name(jul%7), d, plu,
+		snprintf(s, sizeof(s), "%s %s, %d%s %s", DynamicOn, get_day_name(jul%7), d, plu,
 			get_month_name(m));
 	    }
 #endif
@@ -477,7 +585,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_W_OVER
 	    L_W_OVER
 #else
-	    sprintf(s, "%s", get_day_name(jul%7));
+	    snprintf(s, sizeof(s), "%s", get_day_name(jul%7));
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -486,7 +594,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_X_OVER
 	    L_X_OVER
 #else
-	    sprintf(s, "%d", diff);
+	    snprintf(s, sizeof(s), "%d", diff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -495,7 +603,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_Y_OVER
 	    L_Y_OVER
 #else
-	    sprintf(s, "%d", y);
+	    snprintf(s, sizeof(s), "%d", y);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -504,7 +612,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_Z_OVER
 	    L_Z_OVER
 #else
-	    sprintf(s, "%d", y % 100);
+	    snprintf(s, sizeof(s), "%d", y % 100);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -514,14 +622,14 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    L_1_OVER
 #else
 	    if (tdiff == 0)
-		sprintf(s, "%s", L_NOW);
+		snprintf(s, sizeof(s), "%s", DynamicNow);
 	    else if (hdiff == 0)
-		sprintf(s, "%d %s%s %s", mdiff, L_MINUTE, mplu, when);
+		snprintf(s, sizeof(s), "%d %s%s %s", mdiff, DynamicMinute, mplu, when);
 	    else if (mdiff == 0)
-		sprintf(s, "%d %s%s %s", hdiff, L_HOUR, hplu, when);
+		snprintf(s, sizeof(s), "%d %s%s %s", hdiff, DynamicHour, hplu, when);
 	    else
-		sprintf(s, "%d %s%s %s %d %s%s %s", hdiff, L_HOUR, hplu,
-			L_AND, mdiff, L_MINUTE, mplu, when);
+		snprintf(s, sizeof(s), "%d %s%s %s %d %s%s %s", hdiff, DynamicHour, hplu,
+			DynamicAnd, mdiff, DynamicMinute, mplu, when);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -531,9 +639,9 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    L_2_OVER
 #else
 	    if (altmode == '*') {
-		sprintf(s, "%d%c%02d%s", hh, TimeSep, min, pm);
+		snprintf(s, sizeof(s), "%d%c%02d%s", hh, TimeSep, min, pm);
 	    } else {
-		sprintf(s, "%s %d%c%02d%s", L_AT, hh, TimeSep, min, pm);
+		snprintf(s, sizeof(s), "%s %d%c%02d%s", DynamicAt, hh, TimeSep, min, pm);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -545,9 +653,9 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #else
 
 	    if (altmode == '*') {
-		sprintf(s, "%02d%c%02d", h, TimeSep, min);
+		snprintf(s, sizeof(s), "%02d%c%02d", h, TimeSep, min);
 	    } else {
-		sprintf(s, "%s %02d%c%02d", L_AT, h, TimeSep, min);
+		snprintf(s, sizeof(s), "%s %02d%c%02d", DynamicAt, h, TimeSep, min);
 	    }
 #endif
 	    SHIP_OUT(s);
@@ -557,7 +665,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_4_OVER
 	    L_4_OVER
 #else
-	    sprintf(s, "%d", tdiff);
+	    snprintf(s, sizeof(s), "%d", tdiff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -566,7 +674,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_5_OVER
 	    L_5_OVER
 #else
-	    sprintf(s, "%d", adiff);
+	    snprintf(s, sizeof(s), "%d", adiff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -575,7 +683,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_6_OVER
 	    L_6_OVER
 #else
-	    sprintf(s, "%s", when);
+	    snprintf(s, sizeof(s), "%s", when);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -584,7 +692,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_7_OVER
 	    L_7_OVER
 #else
-	    sprintf(s, "%d", hdiff);
+	    snprintf(s, sizeof(s), "%d", hdiff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -593,7 +701,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_8_OVER
 	    L_8_OVER
 #else
-	    sprintf(s, "%d", mdiff);
+	    snprintf(s, sizeof(s), "%d", mdiff);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -602,7 +710,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_9_OVER
 	    L_9_OVER
 #else
-	    sprintf(s, "%s", mplu);
+	    snprintf(s, sizeof(s), "%s", mplu);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -611,7 +719,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_0_OVER
 	    L_0_OVER
 #else
-	    sprintf(s, "%s", hplu);
+	    snprintf(s, sizeof(s), "%s", hplu);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -620,7 +728,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_BANG_OVER
 	    L_BANG_OVER
 #else
-	    sprintf(s, "%s", (tdiff >= 0 ? L_IS : L_WAS));
+            snprintf(s, sizeof(s), "%s", (tdiff >= 0 ? DynamicIs : DynamicWas));
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -629,7 +737,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_AT_OVER
 	    L_AT_OVER
 #else
-	    sprintf(s, "%d%c%02d%s", chh, TimeSep, cmin, cpm);
+	    snprintf(s, sizeof(s), "%d%c%02d%s", chh, TimeSep, cmin, cpm);
 #endif
 	    SHIP_OUT(s);
 	    break;
@@ -638,16 +746,16 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 #ifdef L_HASH_OVER
 	    L_HASH_OVER
 #else
-	    sprintf(s, "%02d%c%02d", ch, TimeSep, cmin);
+	    snprintf(s, sizeof(s), "%02d%c%02d", ch, TimeSep, cmin);
 #endif
 	    SHIP_OUT(s);
 	    break;
 
 	case '_':
 	    if (PsCal == PSCAL_LEVEL2 || PsCal == PSCAL_LEVEL3 || (mode != CAL_MODE && mode != ADVANCE_MODE && !MsgCommand)) {
-		sprintf(s, "%s", NL);
+		snprintf(s, sizeof(s), "%s", NL);
 	    } else {
-		sprintf(s, " ");
+		snprintf(s, sizeof(s), " ");
 	    }
 	    SHIP_OUT(s);
 	    break;
@@ -657,8 +765,13 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 	    break;
 
 	case '"':
-            if (DBufPutc(dbuf, QUOTE_MARKER) != OK) return E_NO_MEM;
-            has_quote = 1;
+            if (DontSuppressQuoteMarkers) {
+                if (DBufPutc(dbuf, '%') != OK) return E_NO_MEM;
+                if (DBufPutc(dbuf, c) != OK) return E_NO_MEM;
+            } else {
+                if (DBufPutc(dbuf, QUOTE_MARKER) != OK) return E_NO_MEM;
+                has_quote = 1;
+            }
 	    break;
 
 	default:
