@@ -6,7 +6,7 @@
 /*  expressions.                                               */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -58,6 +58,7 @@ static int FADusk          (func_info *);
 static int FAbs            (func_info *);
 static int FAccess         (func_info *);
 static int FAmpm           (func_info *);
+static int FTrig           (func_info *);
 static int FIsAny          (func_info *);
 static int FArgs           (func_info *);
 static int FAsc            (func_info *);
@@ -110,6 +111,7 @@ static int FNonomitted     (func_info *);
 static int FNow            (func_info *);
 static int FOrd            (func_info *);
 static int FOstype         (func_info *);
+static int FPad            (func_info *);
 static int FPlural         (func_info *);
 static int FPsmoon         (func_info *);
 static int FPsshade        (func_info *);
@@ -126,6 +128,7 @@ static int FSunset         (func_info *);
 static int FTime           (func_info *);
 static int FTimepart       (func_info *);
 static int FToday          (func_info *);
+static int FTrig           (func_info *);
 static int FTrigback       (func_info *);
 static int FTrigdate       (func_info *);
 static int FTrigdatetime   (func_info *);
@@ -262,6 +265,7 @@ BuiltinFunc Func[] = {
     {   "now",          0,      0,      0,          FNow    },
     {   "ord",          1,      1,      1,          FOrd    },
     {   "ostype",       0,      0,      1,          FOstype },
+    {   "pad",          3,      4,      1,          FPad    },
     {   "plural",       1,      3,      1,          FPlural },
     {   "psmoon",       1,      4,      1,          FPsmoon},
     {   "psshade",      1,      3,      1,          FPsshade},
@@ -279,6 +283,7 @@ BuiltinFunc Func[] = {
     {   "time",         2,      2,      1,          FTime   },
     {   "timepart",     1,      1,      1,          FTimepart },
     {   "today",        0,      0,      0,          FToday  },
+    {   "trig",         0,      NO_MAX, 0,          FTrig },
     {   "trigback",     0,      0,      0,          FTrigback },
     {   "trigdate",     0,      0,      0,          FTrigdate },
     {   "trigdatetime", 0,      0,      0,          FTrigdatetime },
@@ -995,6 +1000,73 @@ static int FOrd(func_info *info)
     return RetStrVal(buf, info);
 }
 
+/***************************************************************/
+/*                                                             */
+/*  FPad - Pad a string to min length                          */
+/*                                                             */
+/*  pad("1", "0", 4) --> "0004"                                */
+/*  pad("1", "0", 4, 1) --> "4000"                             */
+/*  pad("foo", "bar", 7) -> "barbfoo"                          */
+/*                                                             */
+/***************************************************************/
+static int FPad(func_info *info)
+{
+    int r;
+    char *s;
+    DynamicBuffer dbuf;
+    size_t len;
+    size_t wantlen;
+    size_t i;
+
+    ASSERT_TYPE(1, STR_TYPE);
+    ASSERT_TYPE(2, INT_TYPE);
+    if (Nargs == 4) {
+        ASSERT_TYPE(3, INT_TYPE);
+    }
+
+    if (ARG(0).type != STR_TYPE) {
+        r = DoCoerce(STR_TYPE, &ARG(0));
+        if (r != OK) return r;
+    }
+
+    wantlen = ARGV(2);
+    len = strlen(ARGSTR(0));
+    if (len >= wantlen) {
+        DCOPYVAL(RetVal, ARG(0));
+        return OK;
+    }
+
+    if (strlen(ARGSTR(1)) == 0) {
+        return E_BAD_TYPE;
+    }
+
+    if (MaxStringLen > 0 && wantlen > (size_t) MaxStringLen) {
+        return E_STRING_TOO_LONG;
+    }
+
+    DBufInit(&dbuf);
+    s = ARGSTR(1);
+    if (Nargs < 4 || !ARGV(3)) {
+        /* Pad on the LEFT */
+        for (i=0; i<wantlen-len; i++) {
+            DBufPutc(&dbuf, *s++);
+            if (!*s) s = ARGSTR(1);
+        }
+        DBufPuts(&dbuf, ARGSTR(0));
+    } else {
+        /* Pad on the RIGHT */
+        DBufPuts(&dbuf, ARGSTR(0));
+        for (i=0; i<wantlen-len; i++) {
+            DBufPutc(&dbuf, *s++);
+            if (!*s) s = ARGSTR(1);
+        }
+    }
+    r = RetStrVal(DBufValue(&dbuf), info);
+    DBufFree(&dbuf);
+    return r;
+}
+
+
 /***************************************************************/
 /*                                                             */
 /*  FPlural - pluralization function                           */
@@ -2234,10 +2306,10 @@ static int SunStuff(int rise, double cosz, int jul)
     /* Sometimes, we get roundoff error.  Check for "reasonableness" of
        answer. */
     if (rise) {
-	/* Sunrise so close to midnight it wrapped around -- permament light */
+	/* Sunrise so close to midnight it wrapped around -- permanent light */
 	if (hours >= 23) return NO_TIME;
     } else {
-	/* Sunset so close to midnight it wrapped around -- permament light */
+	/* Sunset so close to midnight it wrapped around -- permanent light */
 	if (hours <= 1) return -NO_TIME;
     }
     return hours*60 + mins;
@@ -2411,8 +2483,7 @@ static int FPsshade(func_info *info)
 
     if (!psshade_warned) {
 	psshade_warned = 1;
-	Eprint("psshade() is deprecated; use SPECIAL SHADE instead.");
-        FreshLine = 1;
+	Wprint("psshade() is deprecated; use SPECIAL SHADE instead.");
     }
 
     sprintf(s, "/_A LineWidth 2 div def ");
@@ -2467,8 +2538,7 @@ static int FPsmoon(func_info *info)
     }
     if (!psmoon_warned) {
 	psmoon_warned = 1;
-	Eprint("psmoon() is deprecated; use SPECIAL MOON instead.");
-        FreshLine = 1;
+	Wprint("psmoon() is deprecated; use SPECIAL MOON instead.");
     }
     if (size > 0) {
 	sprintf(sizebuf, "%d", size);
@@ -2971,8 +3041,12 @@ FEvalTrig(func_info *info)
     CreateParser(ARGSTR(0), &p);
     p.allownested = 0;
     r = ParseRem(&p, &trig, &tim, 0);
-    if (r) return r;
+    if (r) {
+        DestroyParser(&p);
+        return r;
+    }
     if (trig.typ != NO_TYPE) {
+        DestroyParser(&p);
 	FreeTrig(&trig);
 	return E_PARSE_ERR;
     }
@@ -2981,7 +3055,7 @@ FEvalTrig(func_info *info)
     } else {
 	/* Hokey... */
 	if (trig.scanfrom != JulianToday) {
-	    Eprint("Warning: SCANFROM is ignored in two-argument form of evaltrig()");
+	    Wprint("Warning: SCANFROM is ignored in two-argument form of evaltrig()");
 	}
 	jul = ComputeTrigger(scanfrom, &trig, &tim, &r, 0);
     }
@@ -2990,6 +3064,7 @@ FEvalTrig(func_info *info)
         jul = -1;
     }
     FreeTrig(&trig);
+    DestroyParser(&p);
     if (r) return r;
     if (jul < 0) {
 	RetVal.type = INT_TYPE;
@@ -3003,3 +3078,59 @@ FEvalTrig(func_info *info)
     }
     return OK;
 }
+
+static int LastTrig = 0;
+static int
+FTrig(func_info *info)
+{
+    Parser p;
+    Trigger trig;
+    TimeTrig tim;
+    int jul;
+    int r;
+    int i;
+
+    RetVal.type = DATE_TYPE;
+    if (Nargs == 0) {
+        RETVAL = LastTrig;
+        return OK;
+    }
+
+    for (i=0; i<Nargs; i++) {
+        ASSERT_TYPE(i, STR_TYPE);
+    }
+
+    RETVAL = 0;
+
+    for (i=0; i<Nargs; i++) {
+        CreateParser(ARGSTR(i), &p);
+        p.allownested = 0;
+        r = ParseRem(&p, &trig, &tim, 0);
+        if (r) {
+            DestroyParser(&p);
+            return r;
+        }
+        if (trig.typ != NO_TYPE) {
+            DestroyParser(&p);
+            FreeTrig(&trig);
+            return E_PARSE_ERR;
+        }
+	jul = ComputeTrigger(trig.scanfrom, &trig, &tim, &r, 0);
+
+        if (r == E_CANT_TRIG) {
+            DestroyParser(&p);
+            FreeTrig(&trig);
+            continue;
+        }
+        if (ShouldTriggerReminder(&trig, &tim, jul, &r)) {
+            LastTrig = jul;
+            RETVAL = jul;
+            DestroyParser(&p);
+            FreeTrig(&trig);
+            return OK;
+        }
+        DestroyParser(&p);
+        FreeTrig(&trig);
+    }
+    return OK;
+}
