@@ -8,15 +8,16 @@
 /*  globals.h and err.h                                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
 #include "config.h"
 
-#include <stdio.h>   /* For defintion of FILE - sigh! */
+#include <stdio.h>   /* For definition of FILE - sigh! */
 #include "types.h"
 #include "custom.h"
+#include "lang.h"
 #define MK_GLOBALS 1
 #include "globals.h"
 #include "err.h"
