@@ -11,6 +11,7 @@
 /*                                                             */
 /***************************************************************/
 
+
 #ifdef MK_GLOBALS
 #undef EXTERN
 #define EXTERN
@@ -24,6 +25,10 @@
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
+EXTERN  FILE *ErrFp;
+
+#include "dynbuf.h"
+#include "lang.h"
 
 #define MAX_TRUSTED_USERS 20
 
@@ -75,6 +80,7 @@ EXTERN  INIT(   long    SysTime, -1L);
 EXTERN	char	const *InitialFile;
 EXTERN	int	FileAccessDate;
 
+EXTERN  INIT(   int     DontSuppressQuoteMarkers, 0);
 EXTERN  INIT(	int 	DontFork, 0);
 EXTERN  INIT(	int 	DontQueue, 0);
 EXTERN  INIT(   int     NumQueued, 0);
@@ -88,13 +94,12 @@ EXTERN  INIT(   int     DefaultColorB, -1);
 EXTERN  INIT(   int     DefaultColorG, -1);
 EXTERN  INIT(   int     SynthesizeTags, 0);
 EXTERN  INIT(   int     ScFormat, SC_AMPM);
-EXTERN  INIT(   int     MaxSatIter, 150);
+EXTERN  INIT(   int     MaxSatIter, 1000);
 EXTERN  INIT(   int     MaxStringLen, MAX_STR_LEN);
 EXTERN  INIT(	char	*FileName, NULL);
 EXTERN	INIT(	int	UseStdin, 0);
 EXTERN  INIT(   int     PurgeMode, 0);
 EXTERN  INIT(   int     PurgeIncludeDepth, 0);
-EXTERN  FILE *ErrFp;
 EXTERN  INIT(   FILE    *PurgeFP,  NULL);
 EXTERN  INIT(   int     NumIfs,    0);
 EXTERN  INIT(   unsigned int IfFlags,   0);
@@ -141,11 +146,6 @@ EXTERN	INIT(	int	  SubsIndent, 0);
 EXTERN	INIT(	char	  *EndSent, ".?!");
 EXTERN	INIT(	char	  *EndSentIg, "\"')]}>");
 
-/* We need the language stuff here... */
-
-#include "lang.h"
-#include "dynbuf.h"
-
 EXTERN DynamicBuffer Banner;
 EXTERN DynamicBuffer LineBuffer;
 EXTERN DynamicBuffer ExprBuf;
@@ -227,3 +227,92 @@ EXTERN	int	MonthIndex[2][12]
 #endif
 ;
 
+EXTERN char *DynamicAgo
+#ifdef MK_GLOBALS
+= L_AGO
+#endif
+;
+EXTERN char *DynamicAm
+#ifdef MK_GLOBALS
+= L_AM
+#endif
+;
+EXTERN char *DynamicAnd
+#ifdef MK_GLOBALS
+= L_AND
+#endif
+;
+EXTERN char *DynamicAt
+#ifdef MK_GLOBALS
+= L_AT
+#endif
+;
+EXTERN char *DynamicFromnow
+#ifdef MK_GLOBALS
+= L_FROMNOW
+#endif
+;
+EXTERN char *DynamicHour
+#ifdef MK_GLOBALS
+= L_HOUR
+#endif
+;
+EXTERN char *DynamicHplu
+#ifdef MK_GLOBALS
+= L_HPLU
+#endif
+;
+EXTERN char *DynamicIs
+#ifdef MK_GLOBALS
+= L_IS
+#endif
+;
+EXTERN char *DynamicMinute
+#ifdef MK_GLOBALS
+= L_MINUTE
+#endif
+;
+EXTERN char *DynamicMplu
+#ifdef MK_GLOBALS
+= L_MPLU
+#endif
+;
+EXTERN char *DynamicNow
+#ifdef MK_GLOBALS
+= L_NOW
+#endif
+;
+EXTERN char *DynamicOn
+#ifdef MK_GLOBALS
+= L_ON
+#endif
+;
+EXTERN char *DynamicPm
+#ifdef MK_GLOBALS
+= L_PM
+#endif
+;
+EXTERN char *DynamicToday
+#ifdef MK_GLOBALS
+= L_TODAY
+#endif
+;
+EXTERN char *DynamicTomorrow
+#ifdef MK_GLOBALS
+= L_TOMORROW
+#endif
+;
+EXTERN char *DynamicWas
+#ifdef MK_GLOBALS
+= L_WAS
+#endif
+;
+
+#define XSTR(x) #x
+#define STRSYSDIR(x) XSTR(x)
+
+EXTERN char *SysDir
+#ifdef MK_GLOBALS
+= STRSYSDIR(SYSDIR);
+#endif
+;
