@@ -7,7 +7,7 @@
 /*  in normal mode.                                            */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -30,10 +30,10 @@
 #endif
 
 #include "types.h"
+#include "globals.h"
 #include "protos.h"
 #include "expr.h"
 #include "err.h"
-#include "globals.h"
 
 /***************************************************************
  *
@@ -154,7 +154,7 @@ void InitRemind(int argc, char const *argv[])
 	    if (FormWidth > 500) FormWidth = 500;
 	}
     }
-    
+
     /* Initialize global dynamic buffers */
     DBufInit(&Banner);
     DBufInit(&LineBuffer);
@@ -232,7 +232,10 @@ void InitRemind(int argc, char const *argv[])
 			Use256Colors = 1;
 		    } else if (x == 2) {
 			UseTrueColors = 1;
-		    }
+		    } else if (x != 0) {
+                        fprintf(ErrFp, "%s: -@n,m: n must be 0, 1 or 2 (assuming 0)\n",
+                                argv[0]);
+                    }
 		}
 		if (*arg == ',') {
 		    arg++;
@@ -241,7 +244,10 @@ void InitRemind(int argc, char const *argv[])
 			TerminalBackground = TERMINAL_BACKGROUND_DARK;
 		    } else if (x == 1) {
 			TerminalBackground = TERMINAL_BACKGROUND_LIGHT;
-		    }
+		    } else {
+                        fprintf(ErrFp, "%s: -@n,m: m must be 0 or 1\n",
+                                argv[0]);
+                    }
 		}
 		break;
 
@@ -459,6 +465,7 @@ void InitRemind(int argc, char const *argv[])
 		DoSimpleCalendar = 1;
 		PsCal = PSCAL_LEVEL1;
 		while (*arg == 'a' || *arg == 'A' ||
+                       *arg == 'q' || *arg == 'Q' ||
 		       *arg == 'p' || *arg == 'P') {
 		    if (*arg == 'a' || *arg == 'A') {
 			DoSimpleCalDelta = 1;
@@ -471,7 +478,9 @@ void InitRemind(int argc, char const *argv[])
 			} else {
 			    PsCal = PSCAL_LEVEL3;
 			}
-		    }
+		    } else if (*arg == 'q' || *arg == 'Q') {
+                        DontSuppressQuoteMarkers = 1;
+                    }
 		    arg++;
 		}
 		PARSENUM(CalMonths, arg);
@@ -672,7 +681,7 @@ void InitRemind(int argc, char const *argv[])
 #ifndef L_USAGE_OVERRIDE
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2021 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2022 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETA VERSION <<<<\n");
 #endif
@@ -698,7 +707,7 @@ void Usage(void)
     fprintf(ErrFp, " -d...  Debug: e=echo x=expr-eval t=trig v=dumpvars l=showline f=tracefiles\n");
     fprintf(ErrFp, " -e     Divert messages normally sent to stderr to stdout\n");
     fprintf(ErrFp, " -b[n]  Time format for cal: 0=am/pm, 1=24hr, 2=none\n");
-    fprintf(ErrFp, " -x[n]  Iteration limit for SATISFY clause (def=150)\n");
+    fprintf(ErrFp, " -x[n]  Iteration limit for SATISFY clause (def=1000)\n");
     fprintf(ErrFp, " -kcmd  Run `cmd' for MSG-type reminders\n");
     fprintf(ErrFp, " -g[dddd] Sort reminders by date, time, priority, and 'timedness'\n");
     fprintf(ErrFp, " -ivar=val Initialize var to val and preserve var\n");
