@@ -11,7 +11,7 @@
 /*  Further corrections by Erik-Jan Vens                       */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -52,9 +52,6 @@
 #define L_AM "am"
 #define L_PM "pm"
 
-/*** The following are only used in dosubst.c ***/
-#ifdef L_IN_DOSUBST
-
 /* Ago and from now */
 #define L_AGO "geleden"
 #define L_FROMNOW "vanaf nu"
@@ -91,15 +88,14 @@ if (tdiff == 0) \
 sprintf(s, L_NOW); \
 else if (hdiff == 0) \
 sprintf(s, "%d %s %s", mdiff, \
-	(mdiff == 1 ? "minuut" : "minuten"), when); \
+        ((mdiff == 1) ? "minuut" : "minuten"), when);   \
 else if (mdiff == 0) \
 sprintf(s, "%d %s %s", hdiff, \
-	(mdiff == 1 ? "uur" : "uren"), when); \
-	else sprintf(s, "%d %s %s %d %s %s", hdiff, \
-		     (hdiff == 1 ? "uur" : "uren"), \
-		     L_AND, mdiff, \
-		     (mdiff == 1 ? "minuut" : "minuten"), \
-		     when);
+        ((hdiff == 1) ? "uur" : "uren"), when); \
+else sprintf(s, "%d %s %s %d %s %s", hdiff, \
+	     (hdiff == 1 ? "uur" : "uren"), \
+	     L_AND, mdiff, \
+	     (mdiff == 1 ? "minuut" : "minuten"), \
+	     when);
 
-#endif /* L_IN_DOSUBST */
 
