@@ -8,7 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*                                                             */
-/*  REMIND is Copyright (C) 1992-2021 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2022 by Dianne Skoll          */
 /*  This file is Copyright (C) 1993 by Laurent Duperval and    */
 /*  Dianne Skoll.                                              */
 /*                                                             */
@@ -51,9 +51,6 @@
 #define L_AM "am"
 #define L_PM "pm"
 
-/*** The following are only used in dosubst.c ***/
-#ifdef L_IN_DOSUBST
-
 /* Ago and from now */
 #define L_AGO "il y a"
 #define L_FROMNOW "dans"
@@ -113,8 +110,6 @@ else if (tdiff < 0) { \
 
 #define L_K_OVER if (altmode == '*') { sprintf(s, "%s, %d%s %s", DayName[jul%7], d, plu, MonthName[m]); } else { sprintf(s, "%s %s, %d%s %s", L_ON, DayName[jul%7], d, plu, MonthName[m]); }
 
-#endif /* L_IN_DOSUBST */
-
 /* The next ones are used only when MK_GLOBALS is set */
 #ifdef MK_GLOBALS
 #define L_ERR_OVERRIDE 1
@@ -232,7 +227,7 @@ EXTERN char *ErrMsg[] =
 #define L_USAGE_OVERRIDE 1
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2021 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (%s version) Copyright 1992-2022 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> BETA VERSION <<<<\n");
 #endif
@@ -258,7 +253,7 @@ void Usage(void)
     fprintf(ErrFp, " -d...  Debug: e=echo x=expr-eval t=trig v=dumpvars l=showline\n");
     fprintf(ErrFp, " -e     Envoyer les messages de stderr à stdout\n");
     fprintf(ErrFp, " -b[n]  Formats de l'heure pour le calendrier: 0=am/pm, 1=24hr, 2=aucun\n");
-    fprintf(ErrFp, " -x[n]  Limite d'itérations pour la clause SATISFY (def=150)\n");
+    fprintf(ErrFp, " -x[n]  Limite d'itérations pour la clause SATISFY (def=1000)\n");
     fprintf(ErrFp, " -kcmd  Exécuter 'cmd' pour les rappels de type MSG\n");
     fprintf(ErrFp, " -g[ddd] Trier les rappels par date, heure et priorité avant d'émettre\n");
     fprintf(ErrFp, " -ivar=val Initialiser var à val et conserver var\n");
