@@ -8,7 +8,7 @@
 /*                                                             */
 /*  This file is part of REMIND.                               */
 /*                                                             */
-/*  REMIND is Copyright (C) 1992-2021 by Dianne Skoll          */
+/*  REMIND is Copyright (C) 1992-2022 by Dianne Skoll          */
 /*  This file is Copyright (C) 1996 by Marco Paganini and      */
 /*  Dianne Skoll.                                              */
 /*                                                             */
@@ -51,9 +51,6 @@
 #define L_AM "am"
 #define L_PM "pm"
 
-/*** The following are only used in dosubst.c ***/
-#ifdef L_IN_DOSUBST
-
 /* Ago and from now */
 #define L_AGO "atras"
 #define L_FROMNOW "adiante"
@@ -138,8 +135,6 @@
     } \
 }
 
-#endif /* L_IN_DOSUBST */
-
 /* The next ones are used only when MK_GLOBALS is set */
 #ifdef MK_GLOBALS
 #define L_ERR_OVERRIDE 1
@@ -257,7 +252,7 @@ EXTERN char *ErrMsg[] =
 #define L_USAGE_OVERRIDE 1
 void Usage(void)
 {
-    fprintf(ErrFp, "\nREMIND %s (versao %s) (C) 1992-2021 Dianne Skoll\n", VERSION, L_LANGNAME);
+    fprintf(ErrFp, "\nREMIND %s (versao %s) (C) 1992-2022 Dianne Skoll\n", VERSION, L_LANGNAME);
 #ifdef BETA
     fprintf(ErrFp, ">>>> VERSAO BETA <<<<\n");
 #endif
@@ -283,7 +278,7 @@ void Usage(void)
     fprintf(ErrFp, " -d...  Debug: e=echo x=expr-eval t=trigger v=dumpvars l=showline\n");
     fprintf(ErrFp, " -e     Desvia mensagens normalmente enviadas a stderr para stdout\n");
     fprintf(ErrFp, " -b[n]  Formato da hora para o cal: 0=am/pm, 1=24hr, 2=nenhum\n");
-    fprintf(ErrFp, " -x[n]  Limite de iteracoes para a clausula SATISFY (default=150)\n");
+    fprintf(ErrFp, " -x[n]  Limite de iteracoes para a clausula SATISFY (default=1000)\n");
     fprintf(ErrFp, " -kcmd  Executa `cmd' para os compromissos com MSG\n");
     fprintf(ErrFp, " -g[ddd] Classifica compromissos por data, hora e prioridade antes de exibir\n");
     fprintf(ErrFp, " -ivar=val Inicializa (e preserva) variavel var com val\n");
