@@ -7,7 +7,7 @@
 /*  Author: Rafa Couto <rafacouto@biogate.com>                 */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -47,9 +47,6 @@
 #define L_AM "am"
 #define L_PM "pm"
 
-/*** The following are only used in dosubst.c ***/
-#ifdef L_IN_DOSUBST
-
 /* Hace y desde hoy */
 #define L_AGO "hace"
 #define L_FROMNOW "desde hoy"
@@ -74,5 +71,3 @@
 #define L_HPLU "s"  
 #define L_MPLU "s"
 
-#endif /* L_IN_DOSUBST */
-
