@@ -6,7 +6,7 @@
 /*  routines, etc.                                             */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -568,6 +568,29 @@ int EvaluateExpr(ParsePtr p, Value *v)
     return OK;
 }
 
+/***************************************************************/
+/*                                                             */
+/*  Wprint - print a warning message.                          */
+/*                                                             */
+/***************************************************************/
+void Wprint(char const *fmt, ...)
+{
+    va_list argptr;
+
+
+    if (FileName) {
+	if (strcmp(FileName, "-"))
+	    (void) fprintf(ErrFp, "%s(%d): ", FileName, LineNo);
+	else
+	    (void) fprintf(ErrFp, "-stdin-(%d): ", LineNo);
+    }
+
+    va_start(argptr, fmt);
+    (void) vfprintf(ErrFp, fmt, argptr);
+    (void) fputc('\n', ErrFp);
+    va_end(argptr);
+    return;
+}
 /***************************************************************/
 /*                                                             */
 /*  Eprint - print an error message.                           */
@@ -582,13 +605,23 @@ void Eprint(char const *fmt, ...)
 
     if (FreshLine && FileName) {
 	FreshLine = 0;
-	if (strcmp(FileName, "-"))
+	if (strcmp(FileName, "-")) {
 	    (void) fprintf(ErrFp, "%s(%d): ", FileName, LineNo);
-	else
+            if (print_callstack(ErrFp)) {
+                (void) fprintf(ErrFp, ": ");
+            }
+        } else {
 	    (void) fprintf(ErrFp, "-stdin-(%d): ", LineNo);
+            if (print_callstack(ErrFp)) {
+                (void) fprintf(ErrFp, ": ");
+            }
+        }
 	if (DebugFlag & DB_PRTLINE) OutputLine(ErrFp);
     } else if (FileName) {
 	fprintf(ErrFp, "       ");
+        if (print_callstack(ErrFp)) {
+            (void) fprintf(ErrFp, ": ");
+        }
     }
 
     va_start(argptr, fmt);
@@ -1112,8 +1145,8 @@ int DoErrMsg(ParsePtr p)
 
 /* The array FoldArray[2][7] contains sample years which begin
    on the specified weekday.  For example, FoldArray[0][2] is a
-   non-leapyear beginning on Wednesday, and FoldArray[1][5] is a
-   leapyear beginning on Saturday.  Used to fold back dates which
+   non-leap year beginning on Wednesday, and FoldArray[1][5] is a
+   leap year beginning on Saturday.  Used to fold back dates which
    are too high for the standard Unix representation.
    NOTE:  This implies that you cannot set BASE > 2001!!!!! */
 static int FoldArray[2][7] = {
