@@ -6,7 +6,7 @@
 /*  the data structures for OMITted dates.                     */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -463,7 +463,7 @@ DoThroughOmit(ParsePtr p, int ystart, int mstart, int dstart)
     end   = Julian(yend,   mend,   dend);
 
     if (end < start) {
-	Eprint("Warning: Swapping dates on OMIT ... THROUGH ... line");
+	Wprint("Warning: Swapping dates on OMIT ... THROUGH ... line");
 	tmp = start;
 	start = end;
 	end = tmp;
