@@ -5,7 +5,7 @@
 /*  Function Prototypes.                                       */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -62,6 +62,7 @@ int EvaluateExpr (ParsePtr p, Value *v);
 int Evaluate (char const **s, Var *locals, ParsePtr p);
 int FnPopValStack (Value *val);
 void Eprint (char const *fmt, ...);
+void Wprint (char const *fmt, ...);
 void OutputLine (FILE *fp);
 void CreateParser (char const *s, ParsePtr p);
 void DestroyParser (ParsePtr p);
@@ -169,3 +170,9 @@ void set_components_from_lat_and_long(void);
 
 char const *get_day_name(int wkday);
 char const *get_month_name(int mon);
+
+int push_call(char const *filename, char const *func, int lineno);
+void clear_callstack(void);
+int have_callstack(void);
+int print_callstack(FILE *fp);
+void pop_call(void);
