@@ -6,7 +6,7 @@
 /*  functions.                                                 */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -32,6 +32,8 @@ typedef struct udf_struct {
     Var *locals;
     char IsActive;
     int nargs;
+    char const *filename;
+    int lineno;
 } UserFunc;
 
 /* The hash table */
@@ -93,6 +95,11 @@ int DoFset(ParsePtr p)
 	DBufFree(&buf);
 	return E_NO_MEM;
     }
+    func->filename = StrDup(FileName);
+    if (!func->filename) {
+        return E_NO_MEM;
+    }
+    func->lineno = LineNo;
     StrnCpy(func->name, DBufValue(&buf), VAR_NAME_LEN);
     DBufFree(&buf);
     if (!Hush) {
@@ -167,7 +174,7 @@ int DoFset(ParsePtr p)
     /* Add the function definition */
     FSet(func);
     if (orig_namelen > VAR_NAME_LEN) {
-	Eprint("Warning: Function name `%s...' truncated to `%s'",
+	Wprint("Warning: Function name `%s...' truncated to `%s'",
 	       func->name, func->name);
     }
     return OK;
@@ -196,6 +203,9 @@ static void DestroyUserFunc(UserFunc *f)
     /* Free the function definition */
     if (f->text) free( (char *) f->text);
 
+    /* Free the filename */
+    if (f->filename) free( (char *) f->filename);
+
     /* Free the data structure itself */
     free(f);
 }
@@ -306,7 +316,11 @@ int CallUserFunc(char const *name, int nargs, ParsePtr p)
     /* Skip the opening bracket, if there's one */
     while (isempty(*s)) s++;
     if (*s == BEG_OF_EXPR) s++;
+    push_call(f->filename, f->name, f->lineno);
     h = Evaluate(&s, f->locals, p);
+    if (h == OK) {
+        pop_call();
+    }
     f->IsActive = 0;
     DestroyLocalVals(f);
     if (DebugFlag &DB_PRTEXPR) {
