@@ -5,7 +5,7 @@
 /*  Useful utility functions.                                  */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -130,24 +130,33 @@ int DateOK(int y, int m, int d)
 
 /* Functions designed to defeat gcc optimizer */
 
-int _private_div(int a, int b) { return a/b; }
-int _private_add_overflow(int result, int b, int old)
+int _private_mul_overflow(int a, int b)
 {
-    if (b > 0 && result < old) return 1;
-    if (b < 0 && result > old) return 1;
-    return 0;
-}
-int _private_sub_overflow(int result, int b, int old)
-{
-    if (b < 0 && result < old) return 1;
-    if (b > 0 && result > old) return 1;
+    double aa = (double) a;
+    double bb = (double) b;
+
+    if (aa*bb > (double) INT_MAX || aa*bb < (double) INT_MIN) {
+        return 1;
+    }
     return 0;
 }
 
-int _private_unminus_overflow(int a, int b)
+int _private_add_overflow(int a, int b)
 {
-    if (a > 0 && b > 0) return 1;
-    if (a < 0 && b < 0) return 1;
+    double aa = (double) a;
+    double bb = (double) b;
+
+    if (aa+bb < (double) INT_MIN) return 1;
+    if (aa+bb > (double) INT_MAX) return 1;
+    return 0;
+}
+int _private_sub_overflow(int a, int b)
+{
+    double aa = (double) a;
+    double bb = (double) b;
+
+    if (aa-bb < (double) INT_MIN) return 1;
+    if (aa-bb > (double) INT_MAX) return 1;
     return 0;
 }
 
@@ -162,3 +171,90 @@ ShellEscape(char const *in, DynamicBuffer *out)
     }
     return OK;
 }
+
+/* Call-stack for printing errors from user-defined functions */
+typedef struct cs_s {
+    struct cs_s *next;
+    char const *filename;
+    char const *func;
+    int lineno;
+} cs;
+
+static cs *callstack = NULL;
+
+static void
+destroy_cs(cs *entry)
+{
+    if (entry->filename) free( (void *) entry->filename);
+    if (entry->func) free( (void *) entry->func);
+    free( (void *) entry);
+}
+
+
+int
+push_call(char const *filename, char const *func, int lineno)
+{
+    cs *entry = NEW(cs);
+    if (!entry) {
+        return E_NO_MEM;
+    }
+    entry->next = NULL;
+    entry->filename = StrDup(filename);
+    entry->func = StrDup(func);
+    entry->lineno = lineno;
+    if (!entry->filename || !entry->func) {
+        destroy_cs(entry);
+        return E_NO_MEM;
+    }
+    entry->next = callstack;
+    callstack = entry;
+    return OK;
+}
+
+void
+clear_callstack(void)
+{
+    cs *entry = callstack;
+    cs *next;
+    while(entry) {
+        next = entry->next;
+        destroy_cs(entry);
+        entry = next;
+    }
+    callstack = NULL;
+}
+
+int
+have_callstack(void)
+{
+    if (callstack) return 1;
+    return 0;
+}
+
+static void
+print_callstack_aux(FILE *fp, cs *entry)
+{
+    if (entry) {
+        print_callstack_aux(fp, entry->next);
+        fprintf(fp, "\n");
+        (void) fprintf(fp, "%s(%d): In function `%s'", entry->filename, entry->lineno, entry->func);
+    }
+}
+
+int
+print_callstack(FILE *fp)
+{
+    print_callstack_aux(fp, callstack);
+    if (callstack) return 1;
+    return 0;
+}
+
+void
+pop_call(void)
+{
+    cs *entry = callstack;
+    if (entry) {
+        callstack = entry->next;
+        destroy_cs(entry);
+    }
+}
