@@ -6,7 +6,7 @@
 /*  user- and system-defined variables.                        */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2022 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -19,12 +19,12 @@
 #include <stdlib.h>
 #include <limits.h>
 #include <errno.h>
+#include <locale.h>
 #include "types.h"
 #include "expr.h"
 #include "globals.h"
 #include "protos.h"
 #include "err.h"
-
 #define UPPER(c) (islower(c) ? toupper(c) : c)
 
 /* The variable hash table */
@@ -40,11 +40,40 @@ static Var *VHashTbl[VAR_HASH_SIZE];
 
 typedef int (*SysVarFunc)(int, Value *);
 
+static double
+strtod_in_c_locale(char const *str, char **endptr)
+{
+    /* Get current locale */
+    char const *loc = setlocale(LC_NUMERIC, NULL);
+    double x;
+
+    /* If it failed, punt */
+    if (!loc) {
+        return strtod(str, endptr);
+    }
+
+    /* Set locale to C */
+    setlocale(LC_NUMERIC, "C");
+
+    x = strtod(str, endptr);
+
+    /* Back to original locale */
+    setlocale(LC_NUMERIC, loc);
+
+    /* If we got an error, try in original locale, but issue a warning */
+    if (**endptr) {
+        x = strtod(str, endptr);
+        if (!**endptr) {
+            Wprint("Accepting \"%s\" for $Latitude/$Longitude, but you should use the \"C\" locale decimal separator \".\" instead", str);
+        }
+    }
+    return x;
+}
+
 static void deprecated_var(char const *var, char const *instead)
 {
     if (DebugFlag & DB_PRTLINE) {
-        Eprint("%s is deprecated; use %s instead", var, instead);
-        FreshLine = 1;
+        Wprint("%s is deprecated; use %s instead", var, instead);
     }
 }
 
@@ -97,9 +126,16 @@ static int latitude_longitude_func(int do_set, Value *val, double *var, double m
     char buf[64];
     double x;
     char *endptr;
+    char const *loc = setlocale(LC_NUMERIC, NULL);
 
     if (!do_set) {
+        if (loc) {
+            setlocale(LC_NUMERIC, "C");
+        }
         snprintf(buf, sizeof(buf), "%f", *var);
+        if (loc) {
+            setlocale(LC_NUMERIC, loc);
+        }
         val->v.str = malloc(strlen(buf)+1);
         if (!val->v.str) return E_NO_MEM;
         strcpy(val->v.str, buf);
@@ -110,9 +146,7 @@ static int latitude_longitude_func(int do_set, Value *val, double *var, double m
         x = (double) val->v.val;
     } else {
         if (val->type != STR_TYPE) return E_BAD_TYPE;
-        errno = 0;
-        x = strtod(val->v.str, &endptr);
-        if (errno) return E_BAD_TYPE;
+        x = strtod_in_c_locale(val->v.str, &endptr);
         if (*endptr) return E_BAD_TYPE;
     }
     if (x < min) return E_2LOW;
@@ -508,7 +542,7 @@ int DoSet (Parser *p)
     if (*DBufValue(&buf) == '$') r = SetSysVar(DBufValue(&buf)+1, &v);
     else r = SetVar(DBufValue(&buf), &v);
     if (buf.len > VAR_NAME_LEN) {
-	Eprint("Warning: Variable name `%.*s...' truncated to `%.*s'",
+	Wprint("Warning: Variable name `%.*s...' truncated to `%.*s'",
 	       VAR_NAME_LEN, DBufValue(&buf), VAR_NAME_LEN, DBufValue(&buf));
     }
     DBufFree(&buf);
@@ -741,7 +775,11 @@ typedef struct {
 /* All of the system variables sorted alphabetically */
 static SysVar SysVarArr[] = {
     /*  name          mod  type              value          min/mal   max  validate*/
+    {"Ago",            1,  STR_TYPE,     &DynamicAgo,          0,      0,    NULL },
+    {"Am",             1,  STR_TYPE,     &DynamicAm,           0,      0,    NULL },
+    {"And",            1,  STR_TYPE,     &DynamicAnd,          0,      0,    NULL },
     {"April",          1,  STR_TYPE,     &DynamicMonthName[3], 0,      0,    NULL },
+    {"At",             1,  STR_TYPE,     &DynamicAt,           0,      0,    NULL },
     {"August",         1,  STR_TYPE,     &DynamicMonthName[7], 0,      0,    NULL },
     {"CalcUTC",        1,  INT_TYPE,     &CalculateUTC,        0,      1,    NULL },
     {"CalMode",        0,  INT_TYPE,     &DoCalendar,          0,      0,    NULL },
@@ -763,11 +801,15 @@ static SysVar SysVarArr[] = {
     {"FoldYear",       1,  INT_TYPE,     &FoldYear,            0,      1,    NULL },
     {"FormWidth",      1,  INT_TYPE,     &FormWidth,           20,     500,  NULL },
     {"Friday",         1,  STR_TYPE,     &DynamicDayName[4],   0,      0,    NULL },
+    {"Fromnow",        1,  STR_TYPE,     &DynamicFromnow,      0,      0,    NULL },
+    {"Hour",           1,  STR_TYPE,     &DynamicHour,         0,      0,    NULL },
+    {"Hplu",           1,  STR_TYPE,     &DynamicHplu,         0,      0,    NULL },
     {"HushMode",       0,  INT_TYPE,     &Hush,                0,      0,    NULL },
     {"IgnoreOnce",     0,  INT_TYPE,     &IgnoreOnce,          0,      0,    NULL },
     {"InfDelta",       0,  INT_TYPE,     &InfiniteDelta,       0,      0,    NULL },
     {"IntMax",         0,  INT_TYPE,     &IntMax,              0,      0,    NULL },
     {"IntMin",         0,  INT_TYPE,     &IntMin,              0,      0,    NULL },
+    {"Is",             1,  STR_TYPE,     &DynamicIs,           0,      0,    NULL },
     {"January",        1,  STR_TYPE,     &DynamicMonthName[0], 0,      0,    NULL },
     {"July",           1,  STR_TYPE,     &DynamicMonthName[6], 0,      0,    NULL },
     {"June",           1,  STR_TYPE,     &DynamicMonthName[5], 0,      0,    NULL },
@@ -785,12 +827,17 @@ static SysVar SysVarArr[] = {
     {"MaxStringLen",   1,  INT_TYPE,     &MaxStringLen,        -1,     ANY,  NULL },
     {"May",            1,  STR_TYPE,     &DynamicMonthName[4], 0,      0,    NULL },
     {"MinsFromUTC",    1,  INT_TYPE,     &MinsFromUTC,         -780,   780,  NULL },
+    {"Minute",         1,  STR_TYPE,     &DynamicMinute,       0,      0,    NULL },
     {"Monday",         1,  STR_TYPE,     &DynamicDayName[0],   0,      0,    NULL },
+    {"Mplu",           1,  STR_TYPE,     &DynamicMplu,         0,      0,    NULL },
     {"NextMode",       0,  INT_TYPE,     &NextMode,            0,      0,    NULL },
     {"November",       1,  STR_TYPE,     &DynamicMonthName[10],0,      0,    NULL },
+    {"Now",            1,  STR_TYPE,     &DynamicNow,          0,      0,    NULL },
     {"NumQueued",      0,  INT_TYPE,     &NumQueued,           0,      0,    NULL },
     {"NumTrig",        0,  INT_TYPE,     &NumTriggered,        0,      0,    NULL },
     {"October",        1,  STR_TYPE,     &DynamicMonthName[9], 0,      0,    NULL },
+    {"On",             1,  STR_TYPE,     &DynamicOn,           0,      0,    NULL },
+    {"Pm",             1,  STR_TYPE,     &DynamicPm,           0,      0,    NULL },
     {"PrefixLineNo",   0,  INT_TYPE,     &DoPrefixLineNo,      0,      0,    NULL },
     {"PSCal",          0,  INT_TYPE,     &PsCal,               0,      0,    NULL },
     {"RunOff",         0,  INT_TYPE,     &RunDisabled,         0,      0,    NULL },
@@ -802,11 +849,14 @@ static SysVar SysVarArr[] = {
     {"SortByTime",     0,  INT_TYPE,     &SortByTime,          0,      0,    NULL },
     {"SubsIndent",     1,  INT_TYPE,     &SubsIndent,          0,      132,  NULL },
     {"Sunday",         1,  STR_TYPE,     &DynamicDayName[6],   0,      0,    NULL },
+    {"SysInclude",     0,  STR_TYPE,     &SysDir,              0,      0,    NULL },
     {"T",              0,  SPECIAL_TYPE, trig_date_func,       0,      0,    NULL },
     {"Td",             0,  SPECIAL_TYPE, trig_day_func,        0,      0,    NULL },
     {"Thursday",       1,  STR_TYPE,     &DynamicDayName[3],   0,      0,    NULL },
     {"TimeSep",        1,  SPECIAL_TYPE, time_sep_func,        0,      0,    NULL },
     {"Tm",             0,  SPECIAL_TYPE, trig_mon_func,        0,      0,    NULL },
+    {"Today",          1,  STR_TYPE,     &DynamicToday,        0,      0,    NULL },
+    {"Tomorrow",       1,  STR_TYPE,     &DynamicTomorrow,     0,      0,    NULL },
     {"Tuesday",        1,  STR_TYPE,     &DynamicDayName[1],   0,      0,    NULL },
     {"Tw",             0,  SPECIAL_TYPE, trig_wday_func,       0,      0,    NULL },
     {"Ty",             0,  SPECIAL_TYPE, trig_year_func,       0,      0,    NULL },
@@ -816,6 +866,7 @@ static SysVar SysVarArr[] = {
     {"UntimedFirst",   0,  INT_TYPE,     &UntimedBeforeTimed,  0,      0,    NULL },
     {"Uw",             0,  SPECIAL_TYPE, today_wday_func,      0,      0,    NULL },
     {"Uy",             0,  SPECIAL_TYPE, today_year_func,      0,      0,    NULL },
+    {"Was",            1,  STR_TYPE,     &DynamicWas,          0,      0,    NULL },
     {"Wednesday",      1,  STR_TYPE,     &DynamicDayName[2],   0,      0,    NULL }
 };
 
@@ -908,9 +959,7 @@ int GetSysVar(char const *name, Value *val)
     /* In "verbose" mode, print attempts to test $RunOff */
     if (DebugFlag & DB_PRTLINE) {
 	if (v->value == (void *) &RunDisabled) {
-	    Eprint("(Security note: $RunOff variable tested.)");
-	    /* Allow further messages from this line */
-	    FreshLine = 1;
+	    Wprint("(Security note: $RunOff variable tested.)");
 	}
     }
     return OK;
